<?php
defined( 'ABSPATH' ) or die( 'No script kiddies please!' );

/**
 * In this file you will find attachment relevant functions.
 * 
 * DEFINED POST TYPES
 * 
 *      define('RML_TYPE_FOLDER', 0);
 *      define('RML_TYPE_COLLECTION', 1);
 *      define('RML_TYPE_GALLERY', 2);
 *
 * ==========================================
 * 
 * Example Szenario #1:
 *   1. User navigates to http://example.com/rml/collection1
 *   2. Use wp_rml_get_by_absolute_path("/collection1") to get the RML_Folder Object
 *   3. (Additional check) $folder->is(RML_TYPE_COLLECTION) to check, if it is a collection.
 *   4. Iterate the childrens with foreach ($folder->children as $value) { }
 *   5. In collection can only be other collections or galleries.
 * 
 *   6. (Additional check) $value->is(RML_TYPE_GALLERY) to check, if it is a gallery.
 *   7. Fetch the IDs with $value->fetchFileIds();
 * 
 * ==========================================
 * 
 * If you want to use more functions look into the RML_Structure Class.
 * You easily get it with RML_Structure::getInstance() (Singleton).
 * 
 * Meaning: Root = Unorganized Pictures
 * 
 * ==========================================
 * 
 * ORDER QUERY
 * 
 * Using the custom order of galleries: In your get_posts()
 * query args use the option "orderby" => "rml" to get the
 * images ordered by custom user order.
 * 
 * ==========================================
 * 
 * CUSTOM FIELDS FOR FOLDERS, COLLECTIONS, GALLERIES, ....
 * 
 * You want create your own custom fields for a rml object?
 * Have a look at the RML_Meta class.
 * 
 * @see inc/metadata/Meta.class.php RML_Meta
 * 
 * ==========================================
 * 
 * EXAMPLES
 * 
 * Filters and actions:
 *      RML/Folder/QueryArgs
 *      RML/Folder/QueryResult
 *      RML/Folder/QueryCountArgs
 *      RML/Folder/Deleted
 *      RML/Folder/Created
 *      RML/Folder/Renamed
 *      RML/Folder/Moved
 *      RML/Item/Moved
 *      RML/Item/MoveFinished
 *      RML/Folder/TreeNode/Icon
 *      RML/Folder/TreeNode/Class
 *      RML/Folder/TreeNode/Content
 *      RML/Folder/TreeNode/Href
 * 
 * Here are a few options for the actions and filters (this are not all)
 * 
 * EXAMPLES IMPLEMENTATION
 * 
 * apply_filters('RML/Folder/QueryArgs', $args)
 *      $args           Arguments for read query WP_Query
 * 
 * apply_filters('RML/Folder/QueryResult', $posts)
 *      $posts          Result of WP_Query
 * 
 * apply_filters('RML/Folder/QueryCountArgs', $args)
 *      $args           Arguments for count query RML_WP_Query_Count class
 * 
 * do_action("RML/Folder/Created", $parent, $name, $type, $id)
 *      $parent         Parent id
 *      $name           New name of the folder
 *      $type           RML_TYPE_... constant
 *      $id             Folder id
 * 
 * do_action('RML/Folder/Renamed', $name, $obj, $oldData)
 *      $name           New name of the folder
 *      $obj            The folder object
 *      $oldData        Row data as array to use for old data
 * 
 * do_action('RML/Folder/Moved', $obj, $id, $ord, $force, $oldData)
 *      $obj            Folder object RML_Folder
 *      $id             ID of the moved folder
 *      $ord            Order number of the moved folder
 *      $force          Boolean
 *      $oldData        Row data as array to use for old data
 * 
 * do_action("RML/Folder/Deleted", $id, $oldData)
 *      $id             ID of the deleted folder
 *      $oldData        Row data as array to use for old data
 * 
 * do_action("RML/Item/Moved", $value, $fromFid, $toFid, $isShortcut)
 *      $value          Post ID of attachment
 *      $fromFid        From folder id
 *      $toFid          To folder id
 *      $isShortcut     Determines if the attachment is as shortcut in the folder
 * 
 * do_action("RML/Item/MoveFinished", $folder, $ids);
 *      $folder         Folder object RML_Folder or null for root
 *      $ids            Array of moved post ids
 */

if (!function_exists('wp_attachment_folder')) {
    /**
     * Returns the folder id of an given attachment or more than one attachment (array). If you pass an array
     * as attachment ids, then the default value does not work, only for single queries.
     * 
     * @param $attachmentId The attachment ID, if you pass an array you get an array of folder IDs
     * @param $default If no folder was found for this, this value is returned for the attachment
     * @return Folder ID or $default or Array
     */
    function wp_attachment_folder($attachmentId, $default = null) {
        return RML_Filter::getInstance()->getAttachmentFolder($attachmentId, $default);
    }
}

if (!function_exists('wp_attachment_order_update')) {
    /**
     * Moves an attachment before or after another attachment in the
     * order table.
     * 
     * @param currentId The attachment which should be moved
     * @param nextId The attachment next to the currentId, if it is
     *               false the currentId should be moved to the end of table.
     * @return boolean
     */
    function wp_attachment_order_update($currentId, $nextId) {
        return RML_Order::getInstance()->update($currentId, $nextId);
    }
}

if (!function_exists('wp_attachment_order_number')) {
    /**
     * Get the order number for a specific attachment.
     * 
     * @param $attachmentId The attachment id
     * @return Int or false
     */
    function wp_attachment_order_number($attachmentId) {
        return RML_Order::getInstance()->getNr($attachmentId);
    }
}

if (!function_exists('wp_rml_move')) {
    /**
     * Move or copy a set of attachments to a specific folder.
     * 
     * If you copy attachments, the actions are also "RML/Item/Move"... but
     * there is a paramter $isShortcut.
     * 
     * @param $to Folder ID, if folder not exists then root will be
     * @param $ids Array of attachment ids
     * @param $supress_validation Supress the permission validation
     * @param $isShortcut Determines, if the ID's are copies
     * @return true or Array with errors
     * 
     * @see RML_Order
     * @see wp_rml_copy
     */
    function wp_rml_move($to, $ids, $supress_validation = false, $isShortcut = false) {
        $folder = RML_Structure::getInstance()->getFolderById($to);
        $folderId = $folder !== null ? $folder->id : _wp_rml_root();
        
        do_action("RML/Item/Move", $folderId, $ids, $folder, $isShortcut);
        
        if ($folder !== null) {
            $errors = $folder->insert($ids, $supress_validation, $isShortcut);
        }else{
            // There can not be errors, expect the "mov" permission (@see RML_Permission)
            if (is_array($ids)) {
                // You can modify the root folder ID, for example for other users media library
    	        $parent = _wp_rml_root();
                
                $validIds = array();
                foreach ($ids as $value) {
                    if ($supress_validation === false) {
                        $errors = RML_Permissions::insert(array(), $value, null);
                        if (count($errors) > 0) {
                            break;
                        }
                    }
                    
                    $validIds[] = $value;
                }
                
                // Get the folder IDs of the attachments
                $foldersToUpdate = wp_attachment_folder($validIds);
                
                // Update the folder
                foreach ($validIds as $value) {
                    _wp_rml_synchronize_attachment($value, $parent, $isShortcut);
                }
                
                $foldersToUpdate[] = $parent;
                do_action("RML/Folder/Inserted", $validIds, $parent, $isShortcut, $foldersToUpdate);
            }
        }
        
        do_action("RML/Item/MoveFinished", $folderId, $ids, $folder, $isShortcut);
        return isset($errors) && count($errors) > 0 ? $errors : true;
    }
}

if (!function_exists('wp_rml_copy')) {
    /**
     * Link/Copy a set of attachments to a specific folder. When the folder
     * has already a given shortcut, the movement for the given attachment will be skipped.
     * 
     * @param $to Folder ID, if folder not exists then root will be
     * @param $ids Array of attachment ids
     * @param $supress_validation Supress the permission validation
     * @return true or Array with errors
     * 
     * @see wp_rml_move
     */
    function wp_rml_copy($to, $ids, $supress_validation = false) {
        return wp_rml_move($to, $ids, $supress_validation, true);
    }
}

if (!function_exists('wp_rml_has_shortcuts')) {
    /**
     * Checks if a given attachment has already a shortcut in a given folder id.
     * 
     * @param $postId The attachment id
     * @param $fid The folder id
     * @return boolean
     */
    function wp_rml_has_shortcut($postId, $fid) {
        global $wpdb;
        $table_name = RML_Core::getInstance()->getTableName("posts");
        $blog_id = get_current_blog_id();
        $sql = $wpdb->prepare("SELECT COUNT(*) FROM $table_name WHERE attachment=%d AND fid=%d AND bid=%d",
            $postId, $fid, $blog_id);
        return $wpdb->get_var($sql) > 0;
    }
}

if (!function_exists('_wp_rml_synchronize_attachment')) {
    /**
     * Synchronizes a result with the realmedialibrary_posts table so on this
     * base there can be made the folder content.
     * 
     * Do not use this directly, instead use the wp_rml_move function.
     * This function should be used in conjunction with _wp_rml_shortcuts_flush.
     * 
     * @param $postId The post ID
     * @param $fid The folder ID
     * @param $isShortcut true = Is shortcut in the given folder, false = Is no shortcut, mainly in this folder
     * @return boolean
     */
    function _wp_rml_synchronize_attachment($postId, $fid, $isShortcut = false) {
        global $wpdb;
        
        // Collect data
        $table_name = RML_Core::getInstance()->getTableName("posts");
        $blog_id = get_current_blog_id();
        $oldFolder = wp_attachment_folder($postId);
        $isShortcut = $isShortcut ? $fid : 0;
        $insert = true;
        
        // Check, if the target folder id has already the attachment in it when creating a shortcut there
        if ($isShortcut) {
            $insert = !wp_rml_has_shortcut($postId, $fid);
        }
        
        // Insert the new attachment relationship
        if ($insert) {
            $sql = $wpdb->prepare("INSERT INTO $table_name (`attachment`, `fid`, `isShortcut`, `bid`)
                VALUES (%d, %d, %d, %d) ON DUPLICATE KEY UPDATE fid=VALUES(fid), isShortcut=VALUES(isShortcut)",
                $postId, $fid, $isShortcut, $blog_id);
            $wpdb->query($sql);
            do_action("RML/Item/Moved", $postId, $oldFolder, $fid, $isShortcut);
            return true;
        }
        return false;
    }
}

if (!function_exists('_wp_rml_shortcuts_flush')) {
    /**
     * This function manages the shortcuts to avoid duplicate entries in the
     * wp_realmedialibrary_posts table.
     * 
     * @param $ids The IDs we have moved/copied
     * @param $fid The folder ID
     * @param $wasProcessedAsShortcut
     */
    function _wp_rml_shortcuts_flush($ids, $fid, $wasProcessedAsShortcut = false) {
        if (count($ids) === 0) {
            return;
        }

        global $wpdb;
        $table_name = RML_Core::getInstance()->getTableName("posts");
        $blog_id = get_current_blog_id();
        $implodeIds = implode(",", $ids);
        if (!$wasProcessedAsShortcut) {
            // If we have moved the files, the _posts table can contain no shortcuts of its IDs
            $sql = $wpdb->prepare("DELETE FROM $table_name WHERE bid = %d AND attachment IN ($implodeIds) AND isShortcut = %d", $blog_id, $fid);
        }
        
        if (isset($sql)) {
            $wpdb->query($sql);
        }
    }
}

/* For dev test purposes, but now use _wp_rml_shortcuts_flush...
SELECT
	r.attachment,
	-- We want to move a file, then delete all entries of this folder
	r.insert_fid AS fid,
	IF (r.action = 'move', 
		0, r.insert_fid
	) AS isShortcut,
	r.bid
FROM (
	SELECT	IF(t.isShortcut=0,'move','copy') AS action, -- General infos
			t.attachment AS attachment,
			t.bid AS bid,
			-- We want insert this
			t.fid AS insert_fid, t.isShortcut AS insert_isShortcut,
			-- There is an existing shortcut in this folder
			p1.fid AS shortcut_fid, p1.isShortcut AS shortcut_isShortcut,
			-- The source folder
			p2.fid AS source_fid, p2.isShortcut AS source_isShortcut
	FROM (
		SELECT
			7302	AS attachment,		-- Try with 7314
			3		AS fid,				-- 3
			0 		AS isShortcut,		-- 3
			1 		AS bid				-- 1
	) AS t
	LEFT JOIN wp_realmedialibrary_posts AS p1 -- There is an existing shortcut in this folder
	ON p1.attachment = t.attachment
		AND p1.isShortcut = t.isShortcut
		AND p1.bid = t.bid
	LEFT JOIN wp_realmedialibrary_posts AS p2 -- The source folder
	ON p2.attachment = t.attachment
		AND p2.isShortcut = 0
		AND p2.bid = t.bid
) AS r
*/
?>