<?php
/**
 * This class handles all hooks for the general filters.
 */

defined( 'ABSPATH' ) or die( 'No script kiddies please!' );

class RML_Filter {
    private static $me = null;
    
    private function __construct() {
    	add_filter('RML/Backend/JS_Localize', array($this, 'localize'));
    }
    
    /**
     * Returns the folder id of an given attachment or more than one attachment (array). If you pass an array
     * as attachment ids, then the default value does not work, only for single queries.
     * 
     * @param $attachmentId The attachment ID, if you pass an array you get an array of folder IDs
     * @param $default If no folder was found for this, this value is returned for the attachment
     * @return Folder ID or $default or Array
     * @api wp_attachment_folder
     */
    public function getAttachmentFolder($attachmentId, $default = null) {
        $isArray = is_array($attachmentId);
        $attachmentId = $isArray ? $attachmentId : array($attachmentId);
        
        if (count($attachmentId) > 0) {
            global $wpdb;
            $attachments_in = implode(",", $attachmentId);
            $table_name = RML_Core::getInstance()->getTableName("posts");
            $blog_id = get_current_blog_id();
            $folders = $wpdb->get_col("SELECT DISTINCT(rmlposts.fid) FROM $table_name AS rmlposts WHERE rmlposts.bid = $blog_id AND rmlposts.attachment IN ($attachments_in) AND rmlposts.isShortcut <= 0");
            if ($isArray) {
                return $folders;
            }else{
                return isset($folders[0]) ? $folders[0] : (($default === null) ? _wp_rml_root() : $default);
            }
        }
        return "";
    }
    
    /**
     * Localize my filter variables for javascripts
     * 
     * @filter RML/Backend/JS_Localize
     */
    public function localize($arr) {
    	//$namesSlugArray = RML_Structure::getInstance()->getView()->namesSlugArray();
        $mode = get_user_option( 'media_library_mode', get_current_user_id() ) ? get_user_option( 'media_library_mode', get_current_user_id() ) : 'grid';
    	
    	// General for filters
    	$arr["ajaxUrl"] = admin_url('admin-ajax.php');
    	//$arr["namesSlug"] = $namesSlugArray; @deprecated since lazy loading
    	$arr["blogId"] = get_current_blog_id();
    	$arr["listMode"] = $mode;
    	$arr["root"] = _wp_rml_root();
    	
    	return $arr;
    }
	
	/**
	 * Changes the SQL query like this way to JOIN the realmedialibrary_posts
	 * table and search for the given folder.
	 * 
	 * @hooked posts_clauses
	 */
	public function posts_clauses($clauses, $query) {
	    global $wpdb;
        $table_name = RML_Core::getInstance()->getTableName("posts");
        $blog_id = get_current_blog_id();
	    
	    // Shortcut destinations
	    $fields = trim($clauses["fields"], ",");
	    $clauses["fields"] = $fields . ", (
	        SELECT GROUP_CONCAT(isShortcut)
	        FROM $table_name AS rmlpostsgc
	        WHERE rmlpostsgc.attachment = ".$wpdb->posts.".ID
	        AND rmlpostsgc.isShortcut > 0
	    ) AS shortcutDests ";
	    
	    if (!empty($query->query_vars['parsed_rml_folder'])) {
	        $folderId = $query->query_vars['parsed_rml_folder'];
	        $root = _wp_rml_root();
	        
	        // Folder relevant data
	        if ($folderId > 0 || $folderId == $root) {
    	        // Change fields
    	        $fields = trim($clauses["fields"], ",");
    	        $clauses["fields"] = $fields . ", rmlposts.fid, rmlposts.isShortcut ";
    	        
    	        // Change join regarding the folder id
    	        $clauses["join"] .= " LEFT JOIN $table_name AS rmlposts ON rmlposts.bid = $blog_id AND rmlposts.attachment = ".$wpdb->posts.".ID ";
    	        
    	        if ($folderId > 0) {
    	            $clauses["join"] .= $wpdb->prepare(" AND rmlposts.fid = %d ", $folderId);
    	            $clauses["where"] .= " AND rmlposts.fid IS NOT NULL ";
    	        }else{
    	            $clauses["where"] .= $wpdb->prepare(" AND (rmlposts.fid IS NULL OR rmlposts.fid = %d) ", $root);
    	        }
	        }
	    }
	    return $clauses;
	}
	
    /**
     * Define a new query option for WP_Query.
     * "rml_folder" integer
     * 
     * @hooked pre_get_posts
     */
    public function pre_get_posts($query) {
        $folder = $this->getFolder($query, RML_Backend::getInstance()->isScreenBase("upload"));
    	
    	if ($folder !== null) {
    	    $query->set('parsed_rml_folder', $folder);
    	}
    }
    
    /**
     * Get folder from different sources.
     * 
     * @return folder id or null
     */
    public function getFolder($query, $fromRequest = false) {
    	$folder = null;
    	
    	if ($query !== null && 
    		($queryFolder = $query->get('rml_folder')) &&
    		isset($queryFolder)) {
    			
	        // Query rml folder from query itself
    		$folder = $queryFolder;
    	}else if(current_user_can("upload_files")) {
    		if ($fromRequest) {
	    		if (isset($_REQUEST["rml_folder"])) {
	    	        // Query rml folder from list mode
	        		$folder = $_REQUEST["rml_folder"];
	        	}else if (isset($_POST["query"]["rml_folder"])) {
	    	        // Query rml folder from grid mode
	    	        $folder = $_POST["query"]["rml_folder"];
	        	}else{
	        		return;
	        	}
    		}
        }else{
    		return null;
    	}
    	return is_numeric($folder) ? $folder : null;
    }
    
    public function ajax_query_attachments_args($query) {
    	$fid = $this->getFolder(null, true);
    	if ($fid !== null) {
    		$query["rml_folder"] = $fid;
    	}
    	return $query;
    }
    
    /**
     * Fix the shortcuts in the folders and create the count for the touched folders.
     * 
     * @hooked RML/Folder/Inserted
     */
    public function folder_inserted($ids, $toFid, $isShortcut, $foldersToUpdate) {
        _wp_rml_shortcuts_flush($ids, $toFid, $isShortcut);
        wp_rml_update_count($foldersToUpdate);
    }
    
    /**
     * Handles the upload to move an attachment directly to a given folder
     * 
     * @hooked add_attachment
     */
    public function add_attachment($postID) {
    	$rmlFolder = isset($_REQUEST["rmlFolder"]) ? $_REQUEST["rmlFolder"] : null;
    	if ($rmlFolder !== null) {
    		$r = wp_rml_move($rmlFolder, array($postID));
    	}
    }
    
    /**
     * Add the attachment ID to the count update
     * 
     * @hooked delete_attachment
     * @see RML_Structure::wp_die
     * @see RML_Structure::$newAttachments
     */
    public function delete_attachment($postID) {
        wp_rml_move(_wp_rml_root(), array($postID)); // Simulate an move to unorganized
        
        global $wpdb;
        $table_name = RML_Core::getInstance()->getTableName("posts");
    	$blog_id = get_current_blog_id();
        $sql = $wpdb->query($wpdb->prepare("DELETE FROM $table_name WHERE attachment = %d AND bid = %d", $postID, $blog_id));
        
        RML_Structure::getInstance()->addNewAttachment($postID);
    }
    
    /**
     * Add a attribute to the ajax output. The attribute represents
     * the folder order number if it is a gallery.
     * 
     * @used order.js
     * @hooked wp_prepare_attachment_for_js
     * @movedpermanently RML_Filter::wp_prepare_attachment_for_js()
     */
    public function wp_prepare_attachment_for_js($response, $attachment, $meta) {
    	$f = wp_attachment_folder($attachment->ID);
    
		// append attribute
		$response['rmlFolderId'] = !empty($f) ? $f : _wp_rml_root();
		
		if (isset($_POST["query"]) &&
				is_array($_POST["query"]) &&
				isset($_POST["query"]["orderby"]) &&
				$_POST["query"]["orderby"] == "rml") {
			$orders = RML_Order::getInstance()->getOrderNumbers($f);
			if (is_array($orders) && isset($orders[$attachment->ID])) {
				$response['rmlGalleryOrder'] = $orders[$attachment->ID];
			}
		}
		
		// Is shortcut property
		if (property_exists($attachment, "shortcutDests")) {
		    $response["shortcutDests"] = explode(",", $attachment->shortcutDests);
		}
		
		// return
		return $response;
	}
	
    /**
     * Create a select option in list table of attachments
     * 
     * @hooked restrict_manage_posts
     * @author MatthiasWeb
     * @since 1.0
     */
    public function restrict_manage_posts() {
        $screen = get_current_screen();
    	if ($screen->id == "upload") {
    		echo '<select name="rml_folder" id="filter-by-rml-folder" class="attachment-filters attachment-filters-rml">
    			' . RML_Structure::getInstance()->optionsFasade(
    						isset($_REQUEST['rml_folder']) ? $_REQUEST['rml_folder'] : "",
    						array()
						) . '
    		</select>&nbsp;';
    	}
    }
    
    public static function getInstance() {
        if (self::$me == null) {
            self::$me = new RML_Filter();
        }
        return self::$me;
    }
}