<?php
/**
 * This class handles all hooks and functions for the structur.
 * If something will print out, this is a fasade-wrapper function
 * for the class RML_View (stored in private $view).
 * 
 * @singleton
 */

defined( 'ABSPATH' ) or die( 'No script kiddies please!' );

class RML_Structure {
    
    private static $me = null;
    
    /**
     * The structure should be accessible within one
     * or more blogs. For those purposes use the wp standard
     * method switch_to_blog();
     * 
     * This is an array of RML_Structure objects.
     * 
     * @see self::getInstance()
     */
    private static $blogs = array();
    
    /**
     * The root folder ID. Can only be set by the constructor!
     * 
     * @see self::newInstance()
     */
    private $root;
    
    /**
     * Array of Databased readed rows
     */
    private $rows;
    /**
     * $rows formed to RML_Folder objects
     */
    private $parsed;
    /**
     * Tree of RML_Folder objects. @see $childrens of RML_Folder object.
     */
    private $tree;
    private $cntAttachments;
    private $view;
    
    /**
     * An array of new attachment ID's which should be updated
     * with the this::updateCountCache method. This includes also
     * deleted attachments. The "new" means the attachments which are changed,
     * but new for the update.
     * 
     * @see this::wp_die
     */
    private $newAttachments = array();
    
    /**
     * Use a valid folder name
     * 
     * @see this::isValidFolderName
     * @see this::createFolder
     * @see RML_Folder::setName
     */
    private $systemReservedFolders = array("/", "..", ".");
    
    /**
     * Checks, if the folder tree is already loaded and do the initial
     * load if needed by an function. So, RML can guarantee lazy loading.
     * 
     * @see this::initialLoad()
     */
    private $hasInitialLoad = false;
    public function initialLoad() {
        if (!$this->hasInitialLoad) {
            $this->hasInitialLoad = true;
            $this->resetData($this->root);
        }
    }

    /**
     * C'tor
     * When starting the structure by singleton getInstance()
     * then fetch all folders with their parents.
     * 
     * @author MatthiasWeb
     * @since 1.0
     */
    private function __construct($root = null) {
        $this->view = new RML_View($this);
        $this->resetData($root, false);
    }
    
    public function resetData($root = null, $fetchData = true) {
        $this->root = $root === null ? _wp_rml_root() : $root;
        $this->rows = array();
        $this->parsed = array();
        $this->tree = array();
        $this->cntAttachments = wp_count_posts('attachment')->inherit;
    
        if ($fetchData) {
            $this->fetch();
        }
    }
    
    /**
     * Fetching all available folders into an array.
     * 
     * @sql example for hierarchical
        SELECT  id,
                name,
                parent 
        FROM    (SELECT * FROM wp_realmedialibrary
                 ORDER BY parent, id) AS products_sorted,
                (SELECT @pv := 'FOLDER_ID') AS tn
        WHERE   FIND_IN_SET(parent, @pv) > 0
        AND     @pv := concat(@pv, ',', id)
     */
    private function fetch() {
        //error_log("Fetching from " . $_SERVER["REQUEST_URI"]);
        global $wpdb;
        
        $table_name = RML_Core::getInstance()->getTableName();
        $where = "";
        $blog_id = null;
        if (is_multisite()) {
            $blog_id = get_current_blog_id();
            $where = " WHERE bid=$blog_id ";
        }
        
        // SELECT fields
        $fields = join(", ", apply_filters("RML/Tree/SQLStatement/SELECT", array(
            // The whole row of the folder
            "tn.*",
            // Count images for this folder
            "IFNULL(tn.cnt, (SELECT COUNT(*)
            	FROM " . RML_Core::getInstance()->getTableName("posts") . " AS rmlpostscnt
            	WHERE rmlpostscnt.fid = tn.id
            )) AS cnt_result"
        )));
        
        // JOINS
        $joins = join(" ", apply_filters("RML/Tree/SQLStatement/JOIN", array()));

        // Full SQL statement filter
        $sqlStatement = apply_filters("RML/Tree/SQLStatement", array("
            SELECT " . $fields . "
            FROM $table_name AS tn
            $where 
            $joins
            ORDER BY parent, ord
        ", $table_name, $blog_id));
        
        $this->rows = $wpdb->get_results($sqlStatement[0]);
        $this->rows = apply_filters("RML/Tree/SQLRows",  $this->rows);
        
        $this->parse();
    }
    
    /**
     * Handle the count cache for the folders. This should avoid
     * a lack SQL subquery which loads data from the postmeta table.
     * 
     * @param $folders Array of folders ID, if null then all folders with cnt = NULL are updated
     * @param $attachments Array of attachments ID, is merged with $folders if given
     * @param $onlyReturn Set to true if you only want the SQL query
     * @return void or SQL query
     */
    public function updateCountCache($folders = null, $attachments = null, $onlyReturn = false) {
        global $wpdb;
        
        $table_name = RML_Core::getInstance()->getTableName();
        
        // Create where statement
        $where = "";
        
        // Update by specific folders
        if (is_array($folders) && count($folders) > 0) {
            $where .= " tn.id IN (" . implode(",", $folders) . ") ";
        }
        
        // Update by attachment IDs, catch all touched 
        if (is_array($attachments) && count($attachments) > 0) {
            $attachments_in = implode(",", $attachments);
            $table_posts = RML_Core::getInstance()->getTableName("posts");
            $blog_id = get_current_blog_id();
            $where .= ($where === "" ? "" : " OR") . " tn.id IN (SELECT DISTINCT(rmlposts.fid) FROM $table_posts AS rmlposts WHERE rmlposts.bid = $blog_id AND rmlposts.attachment IN ($attachments_in)) ";
        }
        
        // Default where statement
        if ($where === "") {
            $where = "tn.cnt IS NULL";
            
            // For multisite, only if no other ID's are given
            $blog_id = null;
            if (is_multisite() || true) {
                $blog_id = get_current_blog_id();
                $where .= " AND bid=$blog_id";
            }
        }
        
        $sqlStatement = "UPDATE $table_name AS tn
            SET cnt = (SELECT COUNT(*)
            	FROM " . RML_Core::getInstance()->getTableName("posts") . " AS rmlpostscnt
            	WHERE rmlpostscnt.fid = tn.id
            )
            WHERE $where";
        if ($onlyReturn) {
            return $sqlStatement;
        }else{
            $wpdb->query($sqlStatement);
        }
    }
    
    /**
     * Reset the count cache for the current blog id
     * 
     * @param $folderId Array If you pass folder id/ids array, only this one will be resetted.
     * @attention The content of the array is not prepared for the statement
     */
    public function resetCountCache($folderId = null) {
        global $wpdb;
        
        $table_name = RML_Core::getInstance()->getTableName();
        $blog_id = get_current_blog_id();
        
        if (is_array($folderId)) {
            $wpdb->query("UPDATE $table_name SET cnt=NULL WHERE id IN (" . implode(",", $folderId) . ")");
        }else{
            $wpdb->query("UPDATE $table_name SET cnt=NULL");
        }
    }
    
    /**
     * Update @ the end of the script execution the count of the given
     * added / deleted attachments.
     * 
     * @uses this::updateCountCache
     */
    public function wp_die() {
        $this->updateCountCache(null, $this->newAttachments);
    }
    
    public function addNewAttachment($id) {
        $this->newAttachments[] = $id;
    }
    
    /**
     * This functions parses the readed rows into folder objects.
     * It also handles the `cnt` cache for the attachments in this folder.
     * 
     * @author MatthiasWeb
     * @since 1.0
     * @see this::updateCountCache
     */
    private function parse() {
        if (!empty($this->rows)) {
            $noCntCache = false;
            foreach ($this->rows as $key => $value) {
                // Check for image cache
                if (is_null($value->cnt)) {
                    $noCntCache = true;
                }
                
                // Craete folder
                $this->parsed[] = new RML_Folder(intval($value->id), intval($value->parent), $value->name,
                    $value->slug, $value->absolute, intval($value->ord), intval($value->type),
                    intval($value->cnt_result), $value
                );
            }
            
            if ($noCntCache) {
                $this->updateCountCache();
            }
        }
        
        // Create the tree
        $folder = null;
        foreach($this->parsed as $key => $category){
            $parent = $category->parent;
            if ($parent > -1) {
                $folder = $this->getFolderByID($parent);
                if ($folder !== null) {
                    $folder->children[] = $category;
                }
            }
        }
        
        $cats_tree = array();
        foreach ($this->parsed as $category) {
            if ($category->parent <= -1) {
                $cats_tree[] = $category;
            }
        }
        $this->tree = $cats_tree;
    }
    
    /**
     * Checks, if a parent is allowed to have the type child.
     * 
     * @param $parent int ID of the folder OR RML_Folder object
     * @param $type int RML_TYPE_...
     * @return boolean
     */
    public function isAllowedTo($parent, $type) {
        if (!$parent instanceof RML_Folder && !$this->isFolder($parent)) {
            return false;
        }
        
        if ($parent instanceof RML_Folder) {
            $parentObj = $parent;
            $parent = $parentObj->id;
        }
        
        if ($parent == -1) {
            // Root directory, no gallerie allowed
            if ($type == 2) {
                return false;
            }
        }else{
            if (!isset($parentObj)) {
                $parentObj = $this->getFolderByID($parent);
            }
            
            // Check, if type is right for the parent folder
            if ($type == 0 && $parentObj->type != 0) {  // If it is a folder, parent must be a folder
                return false;
            } else if ($type == 1 && $parentObj->type == 2) { // If it is a collection, parent must be collection or folder
                return false;
            } else if ($type == 2 && $parentObj->type != 1) { // If it is a gallery, parent may only be a collection
                return false;
            }
        }
        return true;
    }
    
    /**
     * Creates a folder. At first it checks if a folder in parent already exists.
     * Then it checks if the given type is allowed in the parent.
     * 
     * @param $name String Name of the folder
     * @param $parent int ID of the parent (_wp_rml_root() for root)
     * @param $type integer 0|1|2 @see Folder.class.inc
     * @param $restrictions Restrictions for this folder, see RML_Permissions
     *                      The restrictions of the parent folder are also the restrictions
     *                      for the new folder (restrictions ending with ">").
     * @param $supress_validation Supress the permission validation
     * @return  int (ID) when successfully
     *          array with error strings
     * @api wp_rml_create
     */
    public function createFolder($name, $parent, $type, $restrictions = array(), $supress_validation = false) {
        global $wpdb;
        
        $restrictions = is_array($restrictions) ? $restrictions : array();

	    if (is_numeric($type) && $type >= 0 && $type <= 2 && $this->isValidFolderName($name)) {
            // Check if type is allowed in parent folder and the slug is allowed
            if (!$this->isAllowedTo($parent, $type)) {
                return array(__("A folder of this type is not allowed here.", RML_TD));
            }else if ($this->hasChildSlug($parent, $name, false)) {
                return array(__("There is already a folder with this name.", RML_TD));
            }
            
            // Check if other fails are counted
            if (!$supress_validation) {
	            $errors = apply_filters("RML/Validate/Create", array(), $name, $parent, $type);
	            if (count($errors) > 0) {
	                return $errors;
	            }
            }
            
            // Create restrictions from parent
            if ($parent >= 0) {
                $parentFolder = wp_rml_get_by_id($parent, null, true);
                if ($parentFolder instanceof RML_Folder) {
                    $parentRestrictions = $parentFolder->restrictions;
                    foreach ($parentRestrictions as $parentRestriction) {
                        if (substr($parentRestriction, -1) == '>') {
                            $restrictions[] = $parentRestriction;
                        }
                    }
                }
            }
            
            // Create it!
        	$table_name = RML_Core::getInstance()->getTableName();
        	$insert = $wpdb->insert( 
        		$table_name,
        		array( 
        			'parent' => $parent,
        			'slug' => sanitize_title($name, "", "folder"),
        			'name' => $name,
        			'bid' => get_current_blog_id(),
        			'type' => $type,
        			'ord' => $this->getMaxOrder(),
        			'restrictions' => implode(",", array_unique($restrictions))
        		)
        	);

        	if ($insert !== false) {
        	    $id = $wpdb->insert_id;
        	    $this->resetData();
            	do_action("RML/Folder/Created", $parent, $name, $type, $id);
            	return $id;
        	}else{
        	    return array(__("Something went wrong.", RML_TD));
        	}
	    }else{
	        return array(__("Please use a valid folder name.", RML_TD));
	    }
    }
    
    /**
     * Deletes a specific folder.
     * 
     * @param $id int ID of the folder     
     * @param $supress_validation Supress the permission validation
     * @return  int (ID) when successfully
     *          array with error strings
     */
    public function deleteFolder($id, $supress_validation = false) {
        $folder = $this->getFolderByID($id);
        
        if ($folder !== null) {
            // Check if other fails are counted
            if ($supress_validation === false) {
                $errors = apply_filters("RML/Validate/Delete", array(), $id, $folder);
                if (count($errors) > 0) {
                    return $errors;
                }
            }
            
            // Delete folder
            global $wpdb;
            $table_name = RML_Core::getInstance()->getTableName();
            $wpdb->query($wpdb->prepare("DELETE FROM $table_name WHERE id = %d", $id));
            
            $table_name = RML_Core::getInstance()->getTableName("posts");
            $wpdb->query($wpdb->prepare("DELETE FROM $table_name WHERE fid = %d AND bid = %d", $id, get_current_blog_id()));
            
            // Do the action
            $rowData = $folder->getRowData();
            $this->resetData();
            do_action("RML/Folder/Deleted", $id, $rowData);
            
            return true;
        }else{
            return array(__("The given folder does not exist.", RML_TD));
        }
    }
    
    public function optionsFasade($selected, $disabled, $useAll = true) {
        return $this->view->optionsHTML($selected, null, "", "&nbsp;&nbsp;", $useAll, $disabled);
    }
    
    public function isFolder($id) {
        if ($id == -1) {
            return true; // is root directory
        }
        
        return $this->getFolderByID($id) != null;
    }
    
    public function getFolderByID($id) {
        foreach ($this->getParsed() as $folder) {
            if ($folder->id == $id) {
                return $folder;
            }
        }
        return null;
    }
    
    public function getFolderByAbsolutePath($slug) {
        $slug = trim($slug, '/');
        foreach ($this->getParsed() as $folder) {
            if ($folder->absolutePath() == $slug) {
                return $folder;
            }
        }
        return null;
    }
    
    public function getBreadcrumbByID($id) {
        $folder = $this->getFolderByID($id);
        if ($folder === null) {
            return null;
        }
        
        $return = array($folder);
        
        while (true) {
            if ($folder->parent > 0) {
                $folder = $this->getFolderByID($folder->parent);
                if ($folder === null) {
                    return null;
                }else{
                    $return[] = $folder;
                }
            }else{
                break;
            }
        }
        
        return array_reverse($return);
    }
    
    /**
     * Checks, if root has a children with the name.
     * 
     * @param $slug String Slug or Name of folder
     * @param $isSlug boolean Set it to false, if the slug is not santizied
     * @return boolean true/false
     */
    public function hasRootChildSlug($slug, $isSlug = true) {
        if (!$isSlug) {
            $slug = sanitize_title($slug, "", "folder");
        }
        
        foreach ($this->getTree() as $value) {
            if ($value->slug() == $slug) {
                return true;
            }
        }
        return false;
    }
    
    /**
     * Checks, if a Parents folder has a child with the given slug.
     * This function is a wrapper for RML_Structure::hasRootChildSlug
     * and RML_Folder::hasChildSlug. That means, it checks, if a
     * given parent ID is root or folder.
     * 
     * @param $parentID Id of the Parent folder (_wp_rml_root() for root)
     * @param $slug String Slug or Name of folder
     * @param $isSlug boolean Set it to false, if the slug is not santizied
     * @return boolean true/false
     */
    public function hasChildSlug($parentID, $slug, $isSlug = true) {
        if ($parentID == -1) {
            // Check if root has children
            return $this->hasRootChildSlug($slug, $isSlug);
        }else{
            $parentObj = $this->getFolderByID($parentID);
            // Check if children with this name already exists
            if ($parentObj == null || $parentObj->hasChildSlug($slug, $isSlug)) {
                return true;
            }else{
                return false;
            }
        }
    }
    
    /**
     * Checks, if a given folder name is valid
     * 
     * @param $name The folder name
     * @return boolean
     */
    public function isValidFolderName($name) {
        $name = trim($name);
        return strpbrk($name, "\\/?%*:|\"<>") === FALSE && strlen($name) > 0 && !in_array($name, $this->systemReservedFolders);
    }
    
    /**
     * Get the maximal order number of a folder id.
     */
    public function getMaxOrder() {
        global $wpdb;
        $blog_id = get_current_blog_id();
        $table_name = RML_Core::getInstance()->getTableName();
        $order = $wpdb->get_var("SELECT MAX(ord) FROM $table_name WHERE bid=$blog_id");
        return is_numeric($order) ? $order : 8000000;
    }
    
    public function getRows() {
        $this->initialLoad();
        return $this->rows;
    }
    
    public function getParsed() {
        $this->initialLoad();
        return $this->parsed;
    }
    
    public function getTree() {
        $this->initialLoad();
        return $this->tree;
    }
    
    public function getCntAttachments() {
        return $this->cntAttachments;
    }
    
    public function getCntRoot() {
        $cnt = 0;
        foreach ($this->getParsed() as $folder) {
            $cnt += $folder->getCnt();
        }
        $result = $this->getCntAttachments() - $cnt;
        return $result >= 0 ? $result : 0;
    }
    
    public function getView() {
        return $this->view;
    }
    
    /**
     * Get or create an instance of the RML_Structure object
     * depending on the current blog id.
     * 
     * @see self::$blogs
     */
    public static function getInstance() {
        $bid = get_current_blog_id();
        if (!isset(self::$blogs[$bid])) {
            self::$blogs[$bid] = self::newInstance();
        }
        return self::$blogs[$bid];
    }
    
    public static function newInstance($root = null) {
        return new RML_Structure($root);
    }
}

?>